/** 									// -o-
 * Rectangle.pde
 *
 * ----------------------------------------------------------------------
 * Copyright © 2011, David Reeder.  All rights reserved.
 * Please see the LICENSE file in this distribution for license details.
 * ----------------------------------------------------------------------
 */


// ------------------------------------------------ -o--
class Rectangle extends Button  
{
  String  className = "Rectangle"; 

  int  wl, wr, hh, hl; 	// dimentions: width left/right, height high/low
  int  wspan, hspan; 	// width span, height span


  // ------------------------- -o-
  // constructors
  //
  Rectangle() { /*EMPTY*/ } 

  Rectangle(String path_, String color_, int x_, int y_, int wspan_, int hspan_)
  {
    super(path_, color_);
    setRect(x_, y_, wspan_, hspan_);
  }

  Rectangle(String path_, String color_, int x_, int y_, int span_) 	
  {
    super(path_, color_);
    setRect(x_, y_, span_, span_); 	// ie, square
  }


  private void setRect(int x_, int y_, int wspan_, int hspan_) 		// -o-
  {
    wl = x_; 		
    wr = x_ + wspan_; 
    hh = y_;
    hl = y_ + hspan_;

    wspan = wspan_;
    hspan = hspan_;

    checkBounds();

    pixelDiffCountMinimum = min(wl-wr, hh-hl) / _pixelDiffCountRatio;
  }

  protected void checkBounds() 						// -o-
  {
    if (! ((wl >= 0) && (wr < width) && (hh >= 0) && (hl < height)) )
    {
      this.ERRORLN( className + "() is out of bounds -- "
		      + " x=" + wl + ".." + wr + " y=" + hh + ".." + hl );
    }
  }  // checkBounds()



  // ------------------------- -o-
  // helpers
  //
  String toString()  { return toString(""); }

  String toString(String prefix) 
  {
    String  class_desc = className + "(" + osc_path +", "+ colorName +")";
    String  class_info =     "x="+ wl +".."+ wr
			  +" y="+ hh +".."+ hl
			  +" span=" + wspan +"/"+ hspan;


    if ( (prefix != null) && (0 < prefix.length()) ) 
    {
      class_desc = prefix +" = "+ class_desc;
    } 

    return super.toString(class_desc, class_info);

  }  // toString()


  Rectangle display()							
  {
    super.display();
    rect(wl, hh, wspan, hspan);

    if (true == text_enabled) { 
      fill(colorVal, COLOR_SOLID);
      text(text_string, wl+(wspan/2), hh+(hspan/2) +textHOffset); 
    }

    return this;
  }  // display()



  // ------------------------- -o-
  // methods
  //
  boolean isActive() 							// -o-
  {
    if (isDelayed())  { return true; }

    boolean differenceConfirmed = false;
    int diffcount = 0;

    scan_difflist:
    for (int y = hh; y <= hl; y++) {
      for (int x = wl; x <= wr; x++) {
	if (1 == difflist[(y*width)+x])
	{
	  if (pixelDiffCountMinimum < ++diffcount) {
	    differenceConfirmed = true;
	    break scan_difflist;
	  }
        }
      }  // endfor -- x
    }  // endfor -- y


    if (differenceConfirmed) 
    { 	
      trigger_count += 1;
      if (osc_trigger) { oscSend(); }
      setDelayTime(0); 	// positive hit automatically resets delayInterval
    }

    return differenceConfirmed;

  }  // isActive()

}  // class Rectangle

